import { HAP, Logging, Service } from 'homebridge'

export type RegisterConfig<T> = {
  name?: string,
  get: { formatter: (valueMapping: MIoTSpecsResponseValueMapping) => any }
  set?: { formatter: (value: any, previousProperty: T) => any[]; property: string }
}

export type MIoTSpec = { name: string; siid: number, piid: number }
export type MIoTSpecs = MIoTSpec[]
export type MIoTSpecMapping = { [name: string]: MIoTSpec }
export type MIoTSpecGetQuery = MIoTSpec & { did: string }
export type MIoTSpecsGetQuery = MIoTSpecGetQuery[]
export type MIoTSpecSetQuery = MIoTSpec & { did: string; value: any }
export type MIoTSpecsSetQuery = MIoTSpecSetQuery[]
export type MIoTSpecsResponse<T> = MIoTSpec & { did: string; value: T }
export type MIoTSpecsResponseMapping = { [name: string]: MIoTSpecsResponse<any> }
export type MIoTSpecsResponseValueMapping = { [name: string]: any }
export type MIoTDeviceIdentify = { name: string; token: string; address: string; }

export type MIIOSpec = string
export type MIIOSpecMapping = { [name: string]: MIIOSpec }
export type MIIOSpecGetQuery = MIIOSpec[]
export type MIIOSpecResponse = (string | number)[]
export type MIIOSpecResponseValueMapping = { [name: string]: any }

export type DeviceConfig = { name: string; address: string; token: string; }
export type DeviceConfigs = DeviceConfig[]

export class DeviceInstance {
  public id: string
  public did: string
  public timeout: number

  // MIoT
  public miioCall<T extends any> (action: 'get_properties', specQuery: MIoTSpecsGetQuery): MIoTSpecsResponse<T>[]
  public miioCall<T extends any> (action: 'set_properties', specQuery: MIoTSpecsSetQuery): MIoTSpecsResponse<T>[]
  // MIIO
  public miioCall<T extends any> (action: 'get_prop', specQuery: MIIOSpecGetQuery): MIIOSpecResponse
  public miioCall<T extends any> (action: string, specQuery: (string | number)[]): any
}

export type MIoTDeviceProps = {
  identify: MIoTDeviceIdentify
  characteristicsService: Service
}
export class MIoTDevice {
  constructor (props: MIoTDeviceProps)

  public addMIoTSpec (specs: MIoTSpecs): void
  public setMIoTProperty <T extends any> (name: string, value: T): MIoTSpecsResponse<T>[]
  public addMIoTCharacteristicListener(type: any, config: RegisterConfig<MIoTSpecsResponseValueMapping>): void

  public addMIIOSpec (specs: MIIOSpecMapping): void
  public setMIIOProperty <T extends (string | number)> (name: string, value: any[]): any
  public addMIIOCharacteristicListener(type: any, config: RegisterConfig<MIIOSpecResponseValueMapping>): void
}

export type Resolver = (...args: any[]) => void
export class Queue {
  public append(identify: MIoTDeviceIdentify, resolver: Resolver): void
  public resolve(identify: MIoTDeviceIdentify, value: any): void
}
export const SharedQueue: InstanceType<typeof Queue>


export class Device {
  public getInstance(identify: MIoTDeviceIdentify): Promise<DeviceInstance>
}
export const SharedDevice: InstanceType<typeof Device>

export class Foundation {
  public log: Logging
  public hap: HAP
}
export const SharedFoundation: InstanceType<typeof Foundation>
